from unittest import TestCase
from arx.core.plugin import ArxPluginBase
from arx.core.exceptions import ArxValidationError


class MockPlugin(ArxPluginBase):
    attributes = {
        'value1': {
            'type': str,
            'required': True
        },
        'value2': {
            'type': int,
            'required': False
        },
        'value3': {
            'type': list,
            'length': 2,
            'required': False
        }
    }


class TestPluginConfigValidation(TestCase):

    def test_required_check(self):
        with self.assertRaises(ArxValidationError):
            MockPlugin({'value2': 1})

    def test_not_required(self):
        MockPlugin({'value1': 'hi'})

    def test_type_check(self):
        with self.assertRaises(ArxValidationError):
            MockPlugin({'value1': 2})

    def test_length_check(self):
        with self.assertRaises(ArxValidationError):
            MockPlugin({'value1': 'test', 'value3': ['first']})

    def test_unknown_attribute(self):
        with self.assertRaises(ArxValidationError):
            MockPlugin({'value1': 'test', 'value_fake': 'fail'})