from unittest import TestCase
from unittest.mock import MagicMock, patch, Mock
from arx.plugins.ssh import SSHPlugin, RejectPolicy
from arx.core.exceptions import ArxNotConnectedError, ArxExecutionException
from paramiko import SSHClient


class TestSSHPlugin(TestCase):

    def test_connection_error(self):
        ssh_plugin = SSHPlugin(resource='test', hostname='none', username='none')
        with self.assertRaises(ArxNotConnectedError):
            ssh_plugin.send_command('test')

    def test_mock_sending(self):
        def ssh_mock(*args):
            return None, b'test', None
        ssh_plugin = SSHPlugin(resource='test', hostname='none', username='none')
        ssh_plugin.client = SSHClient()
        output = MagicMock()
        stderr = MagicMock()
        output.read = Mock(return_value=b"test")
        stderr.readlines = Mock(return_value=b"")
        ssh_plugin.client.exec_command = Mock(return_value=("", output, stderr))
        result = ssh_plugin.send_command('test')
        expected = 'test'
        self.assertEqual(expected, result)

    def test_mock_sending_error(self):
        def ssh_mock(*args):
            return None, b'test', None
        ssh_plugin = SSHPlugin(resource='test', hostname='none', username='none')
        ssh_plugin.client = SSHClient()
        output = MagicMock()
        stderr = MagicMock()
        output.read = Mock(return_value=b"")
        stderr.readlines = Mock(return_value=["error"])
        ssh_plugin.client.exec_command = Mock(return_value=("", output, stderr))
        with self.assertRaises(ArxExecutionException):
            ssh_plugin.send_command('test')

    def test_disconnect(self):
        ssh_plugin = SSHPlugin(resource='test', hostname='none', username='none')
        ssh_plugin.client = SSHClient()
        ssh_plugin.client.close = Mock()
        ssh_plugin.disconnect()
        ssh_plugin.client.close.assert_called_once()

    def test_connect(self):
        ssh_plugin = SSHPlugin(resource='test', hostname='none', username='none')
        ssh_plugin.client = SSHClient()
        ssh_plugin.client.load_system_host_keys = Mock()
        ssh_plugin.client.set_missing_host_key_policy = Mock()
        ssh_plugin.client.connect = Mock()
        ssh_plugin.connect()
        ssh_plugin.client.load_system_host_keys.assert_called_once()
        ssh_plugin.client.set_missing_host_key_policy.assert_called_once_with(RejectPolicy)
        ssh_plugin.client.connect.assert_called_once()